@@ -108,15 +108,20 @@ int ipkg_install_from_file(ipkg_conf_t *conf, const char *filename)
 	  new_version = pkg_version_str_alloc(pkg);
 
 	  cmp = pkg_compare_versions(old, pkg);
+          if ( (conf->force_downgrade==1) && (cmp > 0) ){     /* We've been asked to allow downgrade  and version is precedent */
+             cmp = -1 ;                                       /* then we force ipkg to downgrade */ 
+                                                              /* We need to use a value < 0 because in the 0 case we are asking to */
+                                                              /* reinstall, and some check could fail asking the "force-reinstall" option */
+          } 
 	  if (cmp > 0) {
-	       ipkg_message(conf, IPKG_NOTICE,
-			    "Not downgrading package %s on %s from %s to %s.\n",
-			    old->name, old->dest->name, old_version, new_version);
-	       pkg->state_want = SW_DEINSTALL;
-	       pkg->state_flag |= SF_OBSOLETE;
-	       free(old_version);
-	       free(new_version);
-	       return 0;
+	         ipkg_message(conf, IPKG_NOTICE,
+			      "Not downgrading package %s on %s from %s to %s.\n",
+			      old->name, old->dest->name, old_version, new_version);
+	         pkg->state_want = SW_DEINSTALL;
+	         pkg->state_flag |= SF_OBSOLETE;
+	         free(old_version);
+	         free(new_version);
+	         return 0;
 	  } else {
 	       free(old_version);
 	       free(new_version);
@@ -146,6 +151,12 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
 	  new_version = pkg_version_str_alloc(new);
 
 	  cmp = pkg_compare_versions(old, new);
+          if ( (conf->force_downgrade==1) && (cmp > 0) ){     /* We've been asked to allow downgrade  and version is precedent */
+	     ipkg_message(conf, IPKG_DEBUG, " Forcing downgrade \n");
+             cmp = -1 ;                                       /* then we force ipkg to downgrade */ 
+                                                              /* We need to use a value < 0 because in the 0 case we are asking to */
+                                                              /* reinstall, and some check could fail asking the "force-reinstall" option */
+          } 
 	  ipkg_message(conf, IPKG_DEBUG, 
 		       "comparing visible versions of pkg %s:"
 		       "\n\t%s is installed "
@@ -168,7 +179,7 @@ ipkg_error_t ipkg_install_by_name(ipkg_conf_t *conf, const char *pkg_name)
 	       return 0;
 	  } else if (cmp < 0) {
 	       new->dest = old->dest;
-	       old->state_want = SW_DEINSTALL;
+	       old->state_want = SW_DEINSTALL;    /* Here probably the problem for bug 1277 */
 	  }
      }
 
@@ -291,6 +302,12 @@ int name_mark_dependencies_for_installation(ipkg_conf_t *conf, const char *pkg_n
 	  new_version = pkg_version_str_alloc(new);
 
 	  cmp = pkg_compare_versions(old, new);
+          if ( (conf->force_downgrade==1) && (cmp > 0) ){     /* We've been asked to allow downgrade  and version is precedent */
+	    ipkg_message(conf, IPKG_DEBUG, " Forcing downgrade ");
+             cmp = -1 ;                                       /* then we force ipkg to downgrade */ 
+                                                              /* We need to use a value < 0 because in the 0 case we are asking to */
+                                                              /* reinstall, and some check could fail asking the "force-reinstall" option */
+          } 
 	  ipkg_message(conf, IPKG_DEBUG, 
 		       "comparing visible versions of pkg %s:"
 		       "\n\t%s is installed "
@@ -641,11 +658,19 @@ static void ipkg_install_pkg_sigint_handler(int sig)
 static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *old_pkg)
 {	  
      if (old_pkg) {
+          char message_out[15];
 	  char *old_version = pkg_version_str_alloc(old_pkg);
 	  char *new_version = pkg_version_str_alloc(pkg);
 	  int cmp = pkg_compare_versions(old_pkg, pkg);
 	  int rc = 0;
 
+          strncpy (message_out,"Upgrading   ",12); 
+          if ( (conf->force_downgrade==1) && (cmp > 0) ){     /* We've been asked to allow downgrade  and version is precedent */
+             cmp = -1 ;                                       /* then we force ipkg to downgrade */ 
+             strncpy (message_out,"Downgrading ",12);                      /* We need to use a value < 0 because in the 0 case we are asking to */
+                                                              /* reinstall, and some check could fail asking the "force-reinstall" option */
+          } 
+
 	  if (cmp > 0) {
 	       ipkg_message(conf, IPKG_NOTICE,
 			    "Not downgrading package %s on %s from %s to %s.\n",
@@ -653,8 +678,8 @@ static int ipkg_install_check_downgrade(ipkg_conf_t *conf, pkg_t *pkg, pkg_t *ol
 	       rc = 1;
 	  } else if (cmp < 0) {
 	       ipkg_message(conf, IPKG_NOTICE,
-			    "Upgrading %s on %s from %s to %s...\n",
-			    pkg->name, old_pkg->dest->name, old_version, new_version);
+			    "%s %s on %s from %s to %s...\n",
+			    message_out, pkg->name, old_pkg->dest->name, old_version, new_version);
 	       pkg->dest = old_pkg->dest;
 	       rc = 0;
 	  } else /* cmp == 0 */ {
