@@ -94,6 +94,8 @@ int pkg_hash_add_from_file(ipkg_conf_t *conf, const char *file_name,
      pkg_t *pkg;
     
      raw = raw_start = read_raw_pkgs_from_file(file_name);
+     if (!raw)
+        return -ENOMEM;
 
      while(*raw){         /* don't worry, we'll increment raw in the parsing function */
 	  pkg = pkg_new();
@@ -202,6 +204,8 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pk
 	       continue;
 	  }
     
+
+/* Could a package provide more than a package ? */
 	  /* now check for supported architecture and satisfaction of constraint_fcn */
 	  {
 	       int max_count = 0;
